/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#pragma once

#include <Core/Block.h>

namespace local_engine
{
namespace DeltaParquetVirtualMeta
{
inline constexpr auto DELTA_INTERNAL_IS_ROW_DELETED = "__delta_internal_is_row_deleted";
inline bool hasMetaColumns(const DB::Block & header)
{
    return header.findByName(DELTA_INTERNAL_IS_ROW_DELETED) != nullptr;
}
inline DB::DataTypePtr getMetaColumnType(const DB::Block & header)
{
    return header.findByName(DELTA_INTERNAL_IS_ROW_DELETED)->type;
}
inline DB::Block removeMetaColumns(const DB::Block & header)
{
    DB::Block new_header;
    for (const auto & col : header)
        if (col.name != DELTA_INTERNAL_IS_ROW_DELETED)
            new_header.insert(col);
    return new_header;
}
}

}